searchData={"items":[{"type":"behaviour","title":"Minch","doc":"A WebSocket client build on top of [`Mint.WebSocket`](https://github.com/elixir-mint/mint_web_socket).","ref":"Minch.html"},{"type":"behaviour","title":"Features - Minch","doc":"- **Reconnects with backoff**\n  - Use `Minch.backoff/2` to calculate an exponential backoff duration.\n- **Handling control frames**\n  - Closes the connection after receiving a `:close` frame.\n  - Replies to server pings automatically.","ref":"Minch.html#module-features"},{"type":"behaviour","title":"Installation - Minch","doc":"The package can be installed by adding `minch` to your list of dependencies in `mix.exs`:\n\n<!-- x-release-please-start-version -->\n\n```elixir\ndef deps do\n  [\n    {:minch, \"~> 0.2.1\"}\n  ]\nend\n```\n\n<!-- x-release-please-end -->","ref":"Minch.html#module-installation"},{"type":"behaviour","title":"Usage - Minch","doc":"","ref":"Minch.html#module-usage"},{"type":"behaviour","title":"Basic example - Minch","doc":"```elixir\ndefmodule EchoClient do\n  use Minch\n\n  @impl true\n  def connect(state) do\n    state.url\n  end\n\n  @impl true\n  def handle_frame(frame, state) do\n    IO.inspect(frame)\n    {:ok, state}\n  end\nend\n\n{:ok, pid} = Minch.start_link(EchoClient, %{url: \"wss://ws.postman-echo.com/raw\"})\nMinch.send_frame(pid, :ping)\n```","ref":"Minch.html#module-basic-example"},{"type":"behaviour","title":"Supervised client - Minch","doc":"```elixir\ndefmodule EchoClient do\n  use Minch\n\n  require Logger\n\n  def start_link(init_arg) do\n    Minch.start_link(__MODULE__, init_arg, name: __MODULE__)\n  end\n\n  @impl true\n  def init(_init_arg) do\n    {:ok, %{connected?: false}}\n  end\n\n  @impl true\n  def connect(_state) do\n    url = \"wss://ws.postman-echo.com/raw\"\n    headers = [{\"authorization\", \"bearer: example\"}]\n    # don't do this in production\n    options = [transport_opts: [{:verify, :verify_none}]]\n    {url, headers, options}\n  end\n\n  @impl true\n  def handle_connect(_response, state) do\n    Logger.info(\"connected\")\n    Process.send_after(self(), :produce, 5000)\n    {:reply, {:text, \"welcome\"}, %{state | connected?: true}}\n  end\n\n  @impl true\n  def handle_disconnect(reason, attempt, state) do\n    Logger.warning(\"disconnected: #{inspect(reason)}\")\n    {:reconnect, Minch.backoff(attempt), %{state | connected?: false}}\n  end\n\n  @impl true\n  def handle_info(:produce, state) do\n    Process.send_after(self(), :produce, 5000)\n    {:reply, {:text, DateTime.utc_now() |> DateTime.to_iso8601()}, state}\n  end\n\n  @impl true\n  def handle_frame(frame, state) do\n    Logger.info(inspect(frame))\n    {:ok, state}\n  end\nend\n```","ref":"Minch.html#module-supervised-client"},{"type":"behaviour","title":"Simple client - Minch","doc":"```elixir\nurl = \"wss://ws.postman-echo.com/raw\"\nheaders = []\n# don't do this in production\noptions = [transport_opts: [{:verify, :verify_none}]]\n\nIO.puts(\"checking ping to #{url}...\")\n\ncase Minch.connect(url, headers, options) do\n  {:ok, pid, ref} ->\n    Minch.send_frame(pid, {:text, to_string(System.monotonic_time())})\n\n    case Minch.receive_frame(ref, 5000) do\n      {:text, start} ->\n        ping =\n          System.convert_time_unit(\n            System.monotonic_time() - String.to_integer(start),\n            :native,\n            :millisecond\n          )\n\n        IO.puts(\"#{ping}ms\")\n\n      :timeout ->\n        IO.puts(\"timeout\")\n    end\n\n    Minch.close(pid)\n\n  {:error, error} ->\n    IO.puts(\"connection error: #{inspect(error)}\")\nend\n```","ref":"Minch.html#module-simple-client"},{"type":"function","title":"Minch.backoff/2","doc":"Calculates an exponential backoff duration.\n\nIt accepts the following options:\n\n  * `:min` - The minimum backoff duration in milliseconds. Defaults to `200`.\n  * `:max` - The maximum backoff duration in milliseconds. Defaults to `30_000`.\n  * `:factor` - The exponent to apply to the attempt number. Defaults to `2`.","ref":"Minch.html#backoff/2"},{"type":"function","title":"Examples - Minch.backoff/2","doc":"iex(8)> for attempt <- 1..20, do: Minch.backoff(attempt)\n    [200, 800, 1800, 3200, 5000, 7200, 9800, 12800, 16200, 20000, 24200, 28800,\n    30000, 30000, 30000, 30000, 30000, 30000, 30000, 30000]\n\n    for attempt <- 1..20, do: Minch.backoff(attempt, min: 100, max: 10000, factor: 1.5)\n    [100, 283, 520, 800, 1118, 1470, 1852, 2263, 2700, 3162, 3648, 4157, 4687, 5238,\n     5809, 6400, 7009, 7637, 8282, 8944]","ref":"Minch.html#backoff/2-examples"},{"type":"type","title":"Minch.callback_result/0","doc":"","ref":"Minch.html#t:callback_result/0"},{"type":"type","title":"Minch.client/0","doc":"","ref":"Minch.html#t:client/0"},{"type":"function","title":"Minch.close/1","doc":"Closes the connection opened by `connect/3`.","ref":"Minch.html#close/1"},{"type":"callback","title":"Minch.connect/1","doc":"Invoked to retrieve the connection details.\n\nSee options for `Mint.HTTP.connect/4` and `Mint.WebSocket.upgrade/5`.","ref":"Minch.html#c:connect/1"},{"type":"function","title":"Minch.connect/3","doc":"Connects to a WebSocket server.\n\nOnce connected, the function will return the connection's PID and the request reference.\n\nUse the `send_frame/2` function to send WebSocket frames to the server.\n\nIncoming WebSocket frames will be sent to the caller's mailbox as a tuple `{:frame, request_ref, frame}`.\nYou can use the `receive_frame/2` function to receive these frames.\n\nUse `Process.monitor/1` to get notified when the connection is closed by the server.","ref":"Minch.html#connect/3"},{"type":"function","title":"Example - Minch.connect/3","doc":"iex> {:ok, pid, ref} = Minch.connect(\"wss://ws.postman-echo.com/raw\", [], transport_opts: [{:verify, :verify_none}])\n    {:ok, _pid, _ref}\n    iex> Minch.send_frame(pid, {:text, \"hello\"})\n    :ok\n    iex> Minch.receive_frame(ref, 5000)\n    {:text, \"hello\"}\n    iex> Minch.close(pid)\n    :ok","ref":"Minch.html#connect/3-example"},{"type":"type","title":"Minch.frame/0","doc":"","ref":"Minch.html#t:frame/0"},{"type":"callback","title":"Minch.handle_connect/2","doc":"Invoked to handle a successful connection.","ref":"Minch.html#c:handle_connect/2"},{"type":"callback","title":"Minch.handle_disconnect/3","doc":"Invoked to handle a disconnect from the server or a failed connection attempt.\n\nReturning `{:reconnect, backoff, state}` will schedule a reconnect after `backoff` milliseconds.","ref":"Minch.html#c:handle_disconnect/3"},{"type":"callback","title":"Minch.handle_error/2","doc":"Invoked to handle internal errors.","ref":"Minch.html#c:handle_error/2"},{"type":"callback","title":"Minch.handle_frame/2","doc":"Invoked to handle an incoming WebSocket frame.","ref":"Minch.html#c:handle_frame/2"},{"type":"callback","title":"Minch.handle_info/2","doc":"Invoked to handle info messages.","ref":"Minch.html#c:handle_info/2"},{"type":"callback","title":"Minch.init/1","doc":"Invoked when the client process is started.","ref":"Minch.html#c:init/1"},{"type":"function","title":"Minch.receive_frame/2","doc":"Receives an incoming WebSocket frame.\n\nSee `connect/3`.","ref":"Minch.html#receive_frame/2"},{"type":"type","title":"Minch.response/0","doc":"","ref":"Minch.html#t:response/0"},{"type":"function","title":"Minch.send_frame/2","doc":"Sends a WebSocket frame.","ref":"Minch.html#send_frame/2"},{"type":"function","title":"Minch.start_link/3","doc":"Starts a `Minch` client process linked to the current process.","ref":"Minch.html#start_link/3"},{"type":"type","title":"Minch.state/0","doc":"","ref":"Minch.html#t:state/0"},{"type":"callback","title":"Minch.terminate/2","doc":"Invoked when the client process is about to exit.","ref":"Minch.html#c:terminate/2"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n## [0.2.1](https://github.com/nmbrone/minch/compare/v0.2.0...v0.2.1) (2025-07-04)","ref":"changelog.html"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* properly handle :done message ([#55](https://github.com/nmbrone/minch/issues/55)) ([a1e597a](https://github.com/nmbrone/minch/commit/a1e597a850aa9f85c141bde4444f202ed4524e19))\n\n## [0.2.0](https://github.com/nmbrone/minch/compare/v0.1.0...v0.2.0) (2025-06-11)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"⚠ BREAKING CHANGES - Changelog","doc":"* remove `{:reconnect, state}` result from `handle_info/2` callback\n* remove public `Minch.Conn`\n* add connection attempt to the `handle_disconnect` callback ([#42](https://github.com/nmbrone/minch/issues/42))\n* handle control frames automatically ([#40](https://github.com/nmbrone/minch/issues/40))\n* replace c:handle_connect/1 with c:handle_connect2 ([#33](https://github.com/nmbrone/minch/issues/33))","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Features - Changelog","doc":"* add `{:close, code, reason, state}` callback result ([#53](https://github.com/nmbrone/minch/issues/53)) ([bc11ac2](https://github.com/nmbrone/minch/commit/bc11ac2f14d9aea328e9a68e13314173fe62e303))\n* add `{:stop, reason, state}` result to some callbacks ([b9430da](https://github.com/nmbrone/minch/commit/b9430dad3eeb1a6961812f47f0efcfc435cb6a89))\n* add `handle_error/2` callback ([e061ab4](https://github.com/nmbrone/minch/commit/e061ab4b89cae3458e7fed1fb0c05c5eb36dc545))\n* add `Minch.backoff/2` ([e061ab4](https://github.com/nmbrone/minch/commit/e061ab4b89cae3458e7fed1fb0c05c5eb36dc545))\n* add connection attempt to the `handle_disconnect` callback ([#42](https://github.com/nmbrone/minch/issues/42)) ([3193661](https://github.com/nmbrone/minch/commit/3193661fa2ecfa507d0c0bfab5b8a5b061a6982d))\n* default implementation for more callbacks ([e061ab4](https://github.com/nmbrone/minch/commit/e061ab4b89cae3458e7fed1fb0c05c5eb36dc545))\n* handle control frames automatically ([#40](https://github.com/nmbrone/minch/issues/40)) ([c41c577](https://github.com/nmbrone/minch/commit/c41c5773076f6d2bc2489d40ca31f953f1fe0733))\n* remove `{:reconnect, state}` result from `handle_info/2` callback ([e061ab4](https://github.com/nmbrone/minch/commit/e061ab4b89cae3458e7fed1fb0c05c5eb36dc545))\n* remove public `Minch.Conn` ([e061ab4](https://github.com/nmbrone/minch/commit/e061ab4b89cae3458e7fed1fb0c05c5eb36dc545))\n* replace c:handle_connect/1 with c:handle_connect2 ([#33](https://github.com/nmbrone/minch/issues/33)) ([4f659b5](https://github.com/nmbrone/minch/commit/4f659b5300dc563226274379cc3645f0330a2840))","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* handle case when the upgrade response is coming in different messages ([e061ab4](https://github.com/nmbrone/minch/commit/e061ab4b89cae3458e7fed1fb0c05c5eb36dc545))\n* handle timeout in simple client ([#45](https://github.com/nmbrone/minch/issues/45)) ([6e67643](https://github.com/nmbrone/minch/commit/6e676432bb21025d2cab3b9fc16baaf2c5e653d0))\n* **TestServer:** fix dialyzer error ([#49](https://github.com/nmbrone/minch/issues/49)) ([1ec1d0e](https://github.com/nmbrone/minch/commit/1ec1d0e02d50df166007137b8df0a92fd7e3f585))","ref":"changelog.html#bug-fixes-1"},{"type":"extras","title":"0.1.0 (2023-10-12) - Changelog","doc":"","ref":"changelog.html#0-1-0-2023-10-12"},{"type":"extras","title":"Features - Changelog","doc":"* initial release ([1bb8bcf](https://github.com/nmbrone/minch/commit/1bb8bcf411aa26145f8ab125986d20ef5f14f994))","ref":"changelog.html#features-1"},{"type":"extras","title":"Miscellaneous Chores - Changelog","doc":"* trigger the release ([be8ba9e](https://github.com/nmbrone/minch/commit/be8ba9eea7a774d4d606740a3104001d8d42bc05))","ref":"changelog.html#miscellaneous-chores"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}